/*C* $Id: tga.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "tga.h"

#include <stdio.h>

void Image::dim(uchar factor)
{
 uchar* p = data;
 for(ulong i=0; i<size(); i++)
  {
   *p = ((ushort)(*p) * (ushort)factor) / (ushort)255;
   p++;
  }
}

bool Tga::load(const char* filename)
{
 FILE* fp = fopen(filename, "r");
 if(!fp) goto error;

 TgaHeader header;
 fread(&header, 18, 1, fp); // THere is a reason for not using sizeof() !!!
 if(ferror(fp)) goto errorClose;

 if(header.imageType != 2) goto errorClose;
 width = header.width; height = header.height;
 alloc();

 fread(data, size(), 1, fp);
 if(ferror(fp)) goto errorFreeClose;

 if(!(header.descriptor & 0x20))
  {
   uchar* newData = new uchar[size()];
   uchar* p = data;
   uchar* q = &newData[size()];
   for(int y=0; y<height; y++)
    {
     q -= width*3;
     memcpy(q, p, width * 3);
     p += width*3;
    }
   delete data;
   data = newData;
  }

 fclose(fp);
 return true;

errorFreeClose:
 free();
errorClose:
 fclose(fp);
error:
 return false;
}

bool Tga::save(const char* filename)
{
 FILE* fp = fopen(filename, "w");
 if(!fp) goto error;

 TgaHeader header;
 header.idSize = 0;
 header.mapType = 0;
 header.imageType = 2;
 header.cmapOrigin = 0;
 header.cmapLength = 0;
 header.cmapEntrySize = 0;
 header.xOrigin = 0;
 header.yOrigin = 0;
 header.width = width;
 header.height = height;
 header.pixelSize = 24;
 header.descriptor = 0x20;

 fwrite(&header, 18, 1, fp);
 fwrite(data, size(), 1, fp);
 
 if(ferror(fp)) goto errorClose;

 fclose(fp);
 return true;

errorClose:
 fclose(fp);
error:
 return false;
}
