/*C* $Id: tga.h,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#ifndef tga_h
#define tga_h

#include <stdlib.h>

typedef unsigned char uchar;
typedef unsigned short ushort;
typedef unsigned long ulong;
typedef signed char schar;
typedef signed short sshort;
typedef signed long slong;

#define PK __attribute__ ((packed));
struct TgaHeader
{
 uchar idSize PK;
 uchar mapType PK;
 uchar imageType PK;
 ushort cmapOrigin PK;
 ushort cmapLength PK;
 uchar cmapEntrySize PK;
 sshort xOrigin PK;
 sshort yOrigin PK;
 ushort width PK;
 ushort height PK;
 uchar pixelSize PK;
 uchar descriptor PK;
};
#undef PK

class Image
{
public:
 Image() { data = NULL; }
 Image(ushort w, ushort h) { width = w; height = h; alloc(); }
 Image(ushort w, ushort h, const uchar* _data) { width = w; height = h; alloc(); memcpy(data, _data, size()); }
 Image(Image &image) { width = image.width; height = image.height; alloc(); memcpy(data, image.data, size()); }
 ~Image() { free(); }
 void free() { if(data) delete data; }
 void alloc() { free(); data = new uchar[size()]; }
 ulong size() { return width * height * 3; }
 void dim(uchar factor);
protected:
 uchar* data;
 ushort width, height;
};

class Tga : public Image
{
public:
 Tga() : Image() {};
 Tga(ushort w, ushort h) : Image(w, h) {};
 Tga(ushort w, ushort h, const uchar* _data) : Image(w, h, _data) {};
 bool load(const char* filename);
 bool save(const char* filename);
};

#endif
