/*C* $Id: tga2spr.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include <stdio.h>
#include <stdlib.h>
#include "vga++/sprite.h"
#include "util.h"

struct t_tga_header
{
 unsigned id_size:8;
 unsigned map_type:8;
 unsigned image_type:8;
};

struct t_tga_header_colormap
{
   signed origin:16;
   signed length:16;
 unsigned entry_size:8;
};

struct t_tga_header_image
{
   signed x_origin:16, y_origin:16;
   signed width:16, height:16;
 unsigned pixel_size:8;
 unsigned descriptor:8;
};

const char* syntax = "Syntax: tga2spr <tga file> <spr file>\n";

int w, h;
unsigned char* data = NULL;
unsigned char* mask = NULL;

bool loadtga(const char* filename)
{
 t_tga_header header; t_tga_header_colormap header_color_map; t_tga_header_image header_image;
 FILE* fp = fopen(filename, "rb");
 if(!fp) return false;
 fseek(fp, 0, SEEK_SET); fread(&header, sizeof(header), 1, fp);
 fseek(fp, 3, SEEK_SET); fread(&header_color_map, sizeof(header_color_map), 1, fp);
 fseek(fp, 8, SEEK_SET); fread(&header_image, sizeof(header_image), 1, fp);
 fseek(fp, 8 + header.id_size, SEEK_SET);
 fseek(fp, header_color_map.length * header_color_map.entry_size, SEEK_CUR);
 if(header.image_type != 2) { fclose(fp); return false; }
 w = header_image.width; h = header_image.height;
 data = new unsigned char[w * h * 3];
 fseek(fp, 18, SEEK_SET); fread(data, w*h*3, 1, fp);
 fclose(fp);
 return true;
}

bool loadmask(const char* filename)
{
 FILE* fp = fopen(filename, "rb");
 if(!fp) return false;
 t_tga_header header; t_tga_header_colormap header_color_map; t_tga_header_image header_image;
 fseek(fp, 0, SEEK_SET); fread(&header, sizeof(header), 1, fp);
 fseek(fp, 3, SEEK_SET); fread(&header_color_map, sizeof(header_color_map), 1, fp);
 fseek(fp, 8, SEEK_SET); fread(&header_image, sizeof(header_image), 1, fp);
 fseek(fp, 8 + header.id_size, SEEK_SET);
 fseek(fp, header_color_map.length * header_color_map.entry_size, SEEK_CUR);
 if(header.image_type != 2) { fclose(fp); return false; }
 if(w != header_image.width || h != header_image.height) return false;
 mask = new unsigned char[w * h * 3];
 fseek(fp, 18, SEEK_SET); fread(mask, w*h*3, 1, fp);
 fclose(fp);
 return true;
}

bool savespr(const char* filename)
{
 FILE* fp = fopen(filename, "wb");
 if(!fp) return false;
 fwrite(&w, sizeof(w), 1, fp);
 fwrite(&h, sizeof(h), 1, fp);
 unsigned char* p = data;
 unsigned char* m = mask;
 int errors[w+1][3];
 for(int x=0; x<w; x++) for(int i=0; i<3; i++) errors[x][i] = 255/5 /2/2;
 for(int y=0; y<h; y++)
  {
   for(int i=0; i<3; i++) errors[0][i] += 255/5 /2/2;
   for(int x=0; x<w; x++)
    {
     int b = *p++ + errors[x][2], g = *p++ + errors[x][1], r = *p++ + errors[x][0];
     int c = 0;

     int got_red = (int)(r*5/255) * 255 / 5;
     int got_green = (int)(g*5/255) * 255 / 5;
     int got_blue = (int)(b*5/255) * 255 / 5;
     int error_red = r - got_red;
     int error_green = g - got_green;
     int error_blue = b - got_blue;
     errors[x+1][0] += errors[x][0] = ((error_red+1) / 2); 
     errors[x+1][1] += errors[x][1] = ((error_green+1) / 2); 
     errors[x+1][2] += errors[x][2] = ((error_blue+1) / 2);
     
     c = (int)(c*6) + (int)(r*5/255);
     c = (int)(c*6) + (int)(g*5/255);
     c = (int)(c*6) + (int)(b*5/255);
     if(m && (((int)*m++ + (int)*m++ + (int)*m++) >> 8)) fputc(0, fp);
     else fputc(c+1, fp);
    }
  }
 fclose(fp);
 return true;
}

int main(int argc, char** argv)
{
 if(argc != 3 && argc != 4) { fprintf(stderr, "%s", syntax); return EXIT_FAILURE; }
 if(!loadtga(argv[1])) { fprintf(stderr, "Error loading %s\n", argv[1]); return EXIT_FAILURE; }
 if(argc == 4) if(!loadmask(argv[3])) Fatal("Error loading mask %s\n", argv[3]);
 if(!savespr(argv[2])) { fprintf(stderr, "Error writing %s\n", argv[2]); return EXIT_FAILURE; }
 if(data) delete data;
}


