/*C* $Id: animation.cc,v 1.3 1997/08/12 17:36:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "src/animation.h"
#include "src/data.h"
#include "util/error.h"
#include "util/file.h"
#include "util/util.h"
#include "vga++/sprite.h"
#include "vga++/vga++.h"
#include <assert.h>

Animation::Animation(int _modes, int _frames, int _repeats, Sprite** _sprites, int** _data)
{
 modes = _modes;
 frames = _frames;
 repeats = _repeats;
 sprites = _sprites;
 data = _data;
 mode = frame = repeat = 0;
 mySprites = false;
}

void Animation::draw(VgaContext* vc, int x, int y, bool advance)
{
 if(!sprites) return;
 if(advance) advanceSubFrame();
 sprites[data? data[mode][frame] : frame]->draw(vc, x, y);
}

void Animation::blur(BlurryVga* vc, int x, int y, bool advance)
{
 if(!sprites) return;
 if(advance) advanceSubFrame();
 sprites[data? data[mode][frame] : frame]->blur(vc, x, y);
}

void Animation::keep(BlurryVga* vc, int x, int y, bool advance)
{
 if(!sprites) return;
 if(advance) advanceSubFrame();
 sprites[data? data[mode][frame] : frame]->keep(vc, x, y);
}

void Animation::advanceFrame()
{
 frame = (frame + 1) % frames;
}

void Animation::advanceSubFrame()
{
 if(repeats && !(repeat = (repeat + 1) % repeats)) frame = (frame + 1) % frames;
}

bool Animation::load(File* f)
{
 assert(f);
 f->read(&w, sizeof(w));
 f->read(&h, sizeof(h));
 if(f->error())
  {
   setError("cannot read header: %y");
   return false;
  }
 if(sprites) delete sprites;
 sprites = new Sprite*[frames];
 for(int i=0; i<frames; i++)
  {
   sprites[i] = new Sprite(w, h, *f);
   if(!sprites[i]->ok) 
    {
     setError("cannot read sprites: %z");
     for(; i; i--) delete sprites[i];
     delete sprites;
     return false;
    }
   sprites[i]->compileAndForget();
  }
 return true;
}

bool Animation::load(const char* filename)
{
 assert(filename);
 File* f = gzFilename(filename)? new GzipPipe(filename, "r") : new File(filename, "r");
 if(!*f)
  {
   setError("%s: %y", filename);
   delete f;
   return false;
  }
 if(!load(f))
  {
   setError("%s: %z", filename);
   delete f;
   return false;
  } 
 if(f->close())
  {
   setError("cannot close %s", filename);
   delete f;
   return false;
  }
 delete f;
 return true;
}


