/*C* $Id: data.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "./animation.h"
#include "./background.h"
#include "./data.h"
#include "./files.h"
#include "./names.h"
#include "./palette.h"
#include "./sprites.h"
#include "util/error.h"
#include "util/util.h"
#include "vga++/font.h"
#include "vga++/sprite.h"
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

//-----------------------------------------------------------------------------
/*
 * The data declerations
 */
Animation* animPlayer = NULL;
Animation* animHat = NULL;
Background* bk = NULL;
Sprites* sprites = NULL;
bool spritesLoaded = false;
Pal* pal = NULL;

PropFont* fontBig = NULL;
PropFont* fontThin = NULL;

//-----------------------------------------------------------------------------
bool loadPalette()
{
 printf("Loading palette.\n");
 if(pal) delete pal;
 pal = new Pal();
 if(!pal->load(libFileName(filePalette)))
  {
   delete pal;
   pal = NULL;
   setError("Cannot load palette: %z");
   return false;
  }
 return true;
}

//-----------------------------------------------------------------------------
bool loadBackground()
{
 printf("Loading background.\n");
 if(bk) delete bk;
 bk = new Background;
 if(!bk->load(libFileName(fileBackground)))
  {
   delete bk;
   bk = NULL;
   setError("Cannot load background: %z");
   return false;
  }
 return true;
}

//-----------------------------------------------------------------------------
bool loadSprites()
{
 printf("Loading sprites.\n");
 if(sprites) delete sprites;
 sprites = new Sprites;
 if(!sprites->load(libFileName(fileSprites)))
  {
   delete sprites;
   sprites = NULL;
   setError("Cannot load sprites: %z");
   return false;
  }
 sprites->compile();

 animPlayer = new Animation(SPRITE_HATMAN_MODES, SPRITE_HATMAN_FRAMES, 2, sprites->getData(), SPRITE_HATMAN);

 return true;
}

//-----------------------------------------------------------------------------
bool loadAnimation()
{
 printf("Loading animation.\n");

 if(animHat) delete animHat;
 animHat = new Animation(1, 24, 0); // 1 mode, 24 frames, 0 repeats (manually advanced)
 if(!animHat->load(libFileName(fileAnimHat)))
  {
   setError("Cannot load animation: %z");
   delete animHat;
   animHat = NULL;
   return false;
  }
 return true;
}

//-----------------------------------------------------------------------------
bool loadFonts()
{
 printf("Loading fonts.\n");

 if(fontBig) delete fontBig;
 fontBig = new PropFont;
 if(!fontBig->load(libFileName(fileFontBig)))
  {
   delete fontBig;
   fontBig = NULL;
   setError("Cannot load fonts: %z");
   return false;
  }
 fontBig->masked();

 if(fontThin) delete fontThin;
 fontThin = new PropFont;
 if(!fontThin->load(libFileName(fileFontThin)))
  {
   delete fontThin;
   fontThin = NULL;
   setError("Cannot load fonts: %z");
   return false;
  }
 fontThin->masked();

 return true;
}

//-----------------------------------------------------------------------------
bool loadData()
{
 printf("Loading...\n");
 if(!loadAnimation()) goto error;
 if(!loadBackground()) goto error;
 if(!loadFonts()) goto error;
 if(!loadPalette()) goto error;
 if(!loadSprites()) goto error;
 return true;

error:
 nonFatal("Error: %z\n");
 return false;
}

//-----------------------------------------------------------------------------
void killData()
{
 if(pal) delete pal;
 if(bk) delete bk;
 if(sprites) delete sprites;
 if(animHat) delete animHat;
 if(fontBig) delete fontBig;
 if(fontThin) delete fontThin;
}
