/*C* $Id: ghost.cc,v 1.3 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "./data.h"
#include "./ghost.h"
#include "./globals.h"
#include "./names.h"
#include "./screen.h"
#include "./sprites.h"
#include "vga++/sprite.h"
#include "vga++/vga++.h"
#include "util/util.h"
#include <stdio.h>
#include <stdlib.h>

int Ghost::flash = 0;

Ghost::Ghost(int _x, int _y, int _defaultSprite)
{
 start.set(_x, _y);
 pos = start;
 state = Alive;

 defaultSprite = _defaultSprite;

 Moving = false;
 dir = None;
}

bool Ghost::move()
{
 if(!Moving) return true;
 if(++q >= sprW)
  {
   Moving = false;
   wrap(pos.x, xInc[dir], lvlW + 1);
   wrap(pos.y, yInc[dir], lvlH + 1);
   return true;
  }
 return false;
}

void Ghost::draw()
{
 Sprite* sprite;
 switch(state)
  {
  case Alive:
   sprite = (*sprites)[defaultSprite];
   break;
  case Scared:
   sprite = (*sprites)[SPRITE_GHOSTSCARED[flash][0]];
   break;
  case Dead:
   sprite = (*sprites)[SPRITE_GHOSTEYES_0_0];
   break;
  default:
   exit(-1);
  }
 Pos screenPos(pos.x * sprW + q * xInc[dir] + xOff, pos.y * sprH + q * yInc[dir] + yOff);
 if(screenPos.x <= -sprW) screenPos.x += vgaW; else if((screenPos.x + sprW) >= vgaW) screenPos.x -= vgaW;
 if(screenPos.y <= -sprH) screenPos.y += vgaH; else if((screenPos.y + sprH) >= vgaH) screenPos.y -= vgaH;
 if(screenPos.x < 0 || screenPos.y < 0)
  {
   if(screenPos.x < 0) sprite->blur(screen, screenPos.x + vgaW, screenPos.y);
   if(screenPos.y < 0) sprite->blur(screen, screenPos.x, screenPos.y + vgaH);
   if(screenPos.x < 0 && screenPos.y < 0) sprite->blur(screen, screenPos.x + vgaW, screenPos.y + vgaH);
  }
 sprite->blur(screen, screenPos.x, screenPos.y); // this is done last, since it increments the animations frame counter
}
  
void Ghost::turnBack()
{
 if(Moving)
  {
   wrap(pos.x, xInc[dir], lvlW + 1);
   wrap(pos.y, yInc[dir], lvlH + 1);
   q = sprW-q;
  }
 dir = behindOf[dir];
}

void Ghost::restart()
{
 pos = start;
 dir = None;
 Moving = false;
 state = Alive;
}
