/*C* $Id: hatlvl.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include <stdio.h>
#include <stdlib.h>
#include "./background.h"
#include "./blocks.h"
#include "./data.h"
#include "./game.h"
#include "./globals.h"
#include "./level.h"
#include "./options.h"
#include "./screen.h"
#include "./spriteslide.h"
#include "util/util.h"
#include "vga++/keycodes.h"
#include "vga++/keys.h"
#include "vga++/vga++blur.h"

static Level* level;

static SpriteSlide* spriteSlide;

static int x = 0, y = 0;

//-----------------------------------------------------------------------------
void killCursor()
{
 level->draw(x, y);
 screen->blur();
}

//-----------------------------------------------------------------------------
void drawCursor()
{
 level->draw(x, y);
 screen->rect(x*sprW + xOff, y*sprH + yOff, sprW, sprH, makeColor(5, 5, 5));
 screen->blur();
}

//-----------------------------------------------------------------------------
static bool isWall(int x, int y)
{
 int sq = level->getSquare(x, y);
 if(BLOCK_IS_WALL(sq)) return true;
 if(BLOCK_IS_BASE(sq)) return true;
 if(BLOCK_IS_SCORE(sq)) return true;
 return false;
}

//-----------------------------------------------------------------------------
static void autoWall()
{
 for(int x=0; x<lvlW; x++) for(int y=0; y<lvlH; y++)
  {
   if(!BLOCK_IS_WALL(level->getSquare(x, y))) continue;
   int i = 0;
   if((x+1) < lvlW && isWall(x+1, y)) i += 1;
   if(x > 0 && isWall(x-1, y)) i += 2;
   if(y > 0 && isWall(x, y-1)) i += 4;
   if((y+1) < lvlH && isWall(x, y+1)) i += 8;
   level->setSquare(x, y, BLOCK_WALL[i]);
  }
 for(int x=0; x<lvlW; x++) for(int y=0; y<lvlH; y++)
  {
   int bRight = 1, bLeft = 2, bUp = 4, bDown = 8;
   if((x+1) < lvlW && level->getSquare(x, y) == BLOCK_WALL[bUp | bRight | bDown]
                   && level->getSquare(x+1, y) == BLOCK_WALL[bUp | bLeft | bDown])
    { level->setSquare(x, y, BLOCK_WALL[bUp | bDown]); level->setSquare(x+1, y, BLOCK_WALL[bUp | bDown]); }
   if((y+1) < lvlH && level->getSquare(x, y) == BLOCK_WALL[bLeft | bDown | bRight]
                   && level->getSquare(x, y+1) == BLOCK_WALL[bLeft | bUp | bRight])
    { level->setSquare(x, y, BLOCK_WALL[bLeft | bRight]); level->setSquare(x, y+1, BLOCK_WALL[bLeft | bRight]); }
  }
 drawCursor();
 screen->blur();
}

//-----------------------------------------------------------------------------
static void save()
{
 FILE* fp;
 if((fp = fopen(options->level, "w")) == NULL) Fatal("cannot save file");
 level->save(fp);
 fclose(fp);
}

//-----------------------------------------------------------------------------
static void setup()
     /*
      * Called before the main loop, and after returning from test mode.
      */
{
 xOff = 0;
 yOff = sprH / 2;
 keyboard->setMode(kmode_repeat);

 bk->copyScreen(*screen);
 level->draw();
 drawCursor();
 screen->update();
 spriteSlide->draw();
}

//-----------------------------------------------------------------------------
void LevelEditor()
{
 spriteSlide = new SpriteSlide(BLOCKS, blockSprites);

 if(!options->level) return;
 level = new Level;
 level->clear();
 level->load(options->level); // yes, ignore the return value (otherwise you can't create new levels)

 setup();

 bool quit = false;
 while(!quit)
  {
   if(keyboard->update())
    {
     KP_UP { killCursor(); wrap(y, -1, lvlH); drawCursor(); }
     KP_DOWN { killCursor(); wrap(y, +1, lvlH); drawCursor(); }
     KP_LEFT { killCursor(); wrap(x, -1, lvlW); drawCursor(); }
     KP_RIGHT { killCursor(); wrap(x, +1, lvlW); drawCursor(); }

     KP_PAGEUP { (*spriteSlide)--; spriteSlide->draw(); }
     KP_PAGEDOWN { (*spriteSlide)++; spriteSlide->draw(); }

     KD_SPACE { level->setSquare(x, y, spriteSlide->sel); drawCursor(); }

     KP_A autoWall();
     KP_S save();
     KP_T { game->play(level); setup(); }

     KP_Q quit = true;
     KP_ESCAPE quit = true;
    }
  } 

 delete spriteSlide;
 delete level;
}

