/*C* $Id: hatman.cc,v 1.3 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "src/background.h"
#include "src/data.h"
#include "src/files.h"
#include "src/game.h"
#include "src/hatlvl.h"
#include "src/hatspr.h"
#include "src/menu.h"
#include "src/options.h"
#include "src/params.h"
#include "src/palette.h"
#include "src/screen.h"
#include "util/error.h"
#include "vga++/keys.h"
#include "vga++/vga++blur.h"
#include <stdio.h>
#include <stdlib.h>

static char startupMessage[] =
 "Hatman version "VERSION", Copyright (c) 1997 by James Pharaoh & Timothy Fisken\n"
 "Hatman comes with ABSOLUTELY NO WARRANTY\n"
 "This is free software, and you are welcome to redistribute it under certain\n"
 "conditions; See the file COPYING for more details, which should have been\n"
 "distributed with the program.\n\n";

void mRedraw()
{
 bk->copyScreen(*screen);
 screen->update();
}

bool mPlay()
{
 runGame(libFileName("games/hatman.game"));
 return true;
}

bool mSprite()
{
 SpriteEditor();
 return true;
}

bool mLevel()
{
 LevelEditor();
 return true;
}

bool mQuit()
{
 return false;
}

//-----------------------------------------------------------------------------
void Menu()
{
 MenuType m(mRedraw);
 m.Add("Play", mPlay);
 m.Add("Quit", mQuit);
 m(vgaW/4, vgaH/2, vgaW/2, vgaH/2, bk);
}

//-----------------------------------------------------------------------------
int main(int argc, char **argv)
/*
 * This is of course, the main function. The idea is that this contains no
 * real code, just calls to functions which do a) initialisation, b) main loop
 * stuff c) shutting down.
 */
{
 // output startup banner
 printf("\n");
 printf("%s", startupMessage);

 initError();
 
 // check parameters
 if(parseOptions(argc, argv))
  {
   // initialise sound
#ifdef SOUND
   if(options->mode == Options::modePlay && options->sound) Effects = new Sound;
#else
   printf("Sound support disabled at compilation.\n");
#endif
   // initialise screen
   screen = new BlurryVga;
   // load data (lib) files
   if(loadData())
    {
     // initialise screen and stuff
     screen->initMode();
     game = new Game;
     keyboard = new KeyboardType;
     keyboard->init();
     pal->Set();
     
     // do it!!!
     switch(options->mode)
      {
      case Options::modePlay:
       if(options->level) game->play(options->level);
       else Menu();
       break;
       
      case Options::modeSprite:
       SpriteEditor();
       break;
       
      case Options::modeLevel:
       LevelEditor();
       break;
      }
     
     // shut down
     delete keyboard;
     delete game;
     killData();
    }
   delete screen;
#ifdef SOUND
   if(Options->mode == OptionsType::modePlay && Options->sound) delete Effects;
#endif
  }
 
 // leave a line, this looks nice
 printf("\n");
 return EXIT_SUCCESS;
}


