/*C* $Id: hatspr.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "src/background.h"
#include "src/colorbar.h"
#include "src/data.h"
#include "src/options.h"
#include "src/screen.h"
#include "src/sprites.h"
#include "src/spriteslide.h"
#include "util/util.h"
#include "vga++/font.h"
#include "vga++/keycodes.h"
#include "vga++/keys.h"
#include "vga++/vga++blur.h"

static int spW, spH; // sprite size
static int ssX, ssY, ssW, ssH; // spriteslide location
static int snX, snY, snW, snH; // sprite number location
static int cbX, cbY, cbW, cbH; // colorbar location
static int eX, eY, eW, eH; // editor coordinates

static int sqW = 10, sqH = 10;

static ColorBar* colorBar;
static Sprites* eSprites;
static SpriteSlide* spriteSlide;

static int x = 0, y = 0;

static void drawBoxes()
{
 for(int x=0; x<spW; x++) for(int y=0; y<spH; y++)
  if((*eSprites)[spriteSlide->sel]->getPixel(x, y))
   screen->fillBox(x * sqW + 1, y * sqH + 1, sqW, sqH - 1, (*eSprites)[spriteSlide->sel]->getPixel(x,y));
}

static void drawGrid()
{
 screen->grid(eX, eY, /*sqX*/ spW, /*sqY*/ spH, sqW, sqH, makeColor(5, 5, 5));
 screen->rect((x) * sqW, (y) * sqH, sqW + 1, sqH + 1, makeColor(5, 0, 0));
}

static void drawNumber()
{
 defaultFont.masked();
 defaultFont.color(makeColor(5, 5, 5));
 defaultFont.setHAlign(Font::Left);
 defaultFont.restoreAndWritefAndUpdate(screen, snX, snY, "%.2X", spriteSlide->sel);
}

static void restoreEditor()
{
 screen->restore(eX, eY, eW, eH);
}

static void updateEditor()
{
 screen->update(eX, eY, eW, eH);
}

static void redrawGrid()
{
 drawGrid();
 updateEditor();
}

static void drawEditor()
{
 restoreEditor();
 drawBoxes();
 drawGrid();
 updateEditor();
}

void SpriteEditor()
{
 if(!options->level) Fatal("Sprite editor requires filename.\n");
 
 eSprites = new Sprites();
 if(!eSprites->load(options->level))
  Fatal("Sprite load failed\n");

 spW = (*eSprites)[0]->getWidth(); spH = (*eSprites)[0]->getHeight(); // sprite size
 ssW = spW; ssX = vgaW - ssW; ssY = 0; ssH = vgaH; // spriteslide location
 snW = 16; snX = ssX - snW; snH = 8; snY = 0; // sprite number location
 cbW = snW; cbX = vgaW - ssW - cbW; cbH = vgaH - snH; cbY = snY + snH; // colorbar location

 eX = 0; eY = 0;
 eW = (vgaW - ssW - cbW) <? (spW * sqW + 1); eH = (vgaH) <? (spH * sqH + 1); // editor location

 colorBar = new ColorBar(cbX, cbY, cbW, cbH);
 spriteSlide = new SpriteSlide(eSprites->getCount(), eSprites->getData());
 
 bool quit = false;
 
#define SetTranslate() keyboard->setMode(kmode_repeat)
 SetTranslate();

 bk->copyScreen(*screen);
 screen->update();

 colorBar->update();
 spriteSlide->draw();
 drawEditor();
 drawNumber();

 while(!quit)
  {
   keyboard->update();
   KP_UP
    { wrap(y, -1, spH); redrawGrid(); }
   KP_DOWN
    { wrap(y, +1, spH); redrawGrid(); }
   KP_RIGHT
    { wrap(x, +1, spW); redrawGrid(); }
   KP_LEFT
    { wrap(x, -1, spW); redrawGrid(); }
   KP_Q quit = true;
   KD_REMOVE
    { (*eSprites)[spriteSlide->sel]->setPixel(x, y, 0); drawEditor(); spriteSlide->update(0); } 
   KD_SPACE
    { (*eSprites)[spriteSlide->sel]->setPixel(x, y, colorBar->cur); drawEditor(); spriteSlide->update(0); }
   KP_H
    { (*eSprites)[spriteSlide->sel]->flipH(); drawEditor(); spriteSlide->update(0); }
   KP_V
    { (*eSprites)[spriteSlide->sel]->flipV(); drawEditor(); spriteSlide->update(0); }
   KP_R
    { (*eSprites)[spriteSlide->sel]->rotate(); drawEditor(); spriteSlide->update(0); }
   KP_S
    { eSprites->save(options->level); }
   KP_KEYPADPLUS 
    {
     wrap(spriteSlide->sel, +1, eSprites->getCount());
     spriteSlide->draw();
     drawEditor();
     drawNumber();
    }
   KP_KEYPADMINUS
    {
     wrap(spriteSlide->sel, -1, eSprites->getCount());
     spriteSlide->draw();
     drawEditor();
     drawNumber();
    }
   KP_PAGEUP
    {
     KD_LSHIFT wrap(colorBar->cur, -16, 256);
     else wrap(colorBar->cur, -1, 256);
     colorBar->update();
    }
   KP_PAGEDOWN
    {
     KD_LSHIFT wrap(colorBar->cur, +16, 256);
     else wrap(colorBar->cur, +1, 256);
     colorBar->update();
    }
   KP_0
    { colorBar->cur = makeColor(0, 0, 0); colorBar->update(); }
   KP_1
    { colorBar->cur = makeColor(5, 5, 5); colorBar->update(); }
  }

 delete spriteSlide;
 delete colorBar;
 delete eSprites;
}
