/*C* $Id: level.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "./background.h"
#include "./blocks.h"
#include "./data.h"
#include "./level.h"
#include "./screen.h"
#include "./sprites.h"
#include "util/file.h"
#include "util/util.h"
#include "vga++/sprite.h"
#include "vga++/vga++blur.h"

//-----------------------------------------------------------------------------
Level::Level()
{
}

//-----------------------------------------------------------------------------
Level& Level::operator=(const Level& level)
{
 memcpy(data, level.data, sizeof(level.data));
 dots = level.dots;
 return *this;
}

//-----------------------------------------------------------------------------
bool Level::load(File* f)
/*
 * Loads a level from a FILE*
 */
{
 bool ret;
 ret = f->read(&data[0][0], sizeof(data))? true : false;
 // set up the dot count & stuff
 dots = 0;
 for(int x=0; x<lvlW; x++) for(int y=0; y<lvlH; y++)
  {
   // the next line stops data from ever being invalid once loaded
   data[x][y] %= BLOCKS;
   if(BLOCK_IS_DOT(data[x][y])) dots++;
  }
 return ret;
}

//-----------------------------------------------------------------------------
bool Level::load(const char* filename)
/*
 * Loads a level from the file named
 */
{
 File* f = new File(filename, "r");
 if(!*f) goto error;
 if(!load(f)) goto error;
 delete f;
 return true;
error:
 delete f;
 nonFatal("Error loading level from %s.\n", filename);
 return false;
}

//-----------------------------------------------------------------------------
void Level::getGhosts(Collection<Ghost>* Ghosts)
/*
 * This scans the level and creates a GhostType object for each ghost it finds.
 * It is actuallyy extremely nasty, because it contains no error checking
 * whatsoever, so too many or too few ghosts will probably piss it off a bit...
 */
{
 for(int x=0; x<lvlW; x++) for(int y=0; y<lvlH; y++)
  {
   if(data[x][y] >= BLOCK_GHOST_0 && data[x][y] < BLOCK_GHOST_LAST)
    {
     Ghosts->add(new Ghost(x, y, blockSprites[data[x][y]]));
     clear(x, y);
    }
  }
}

//-----------------------------------------------------------------------------
bool Level::save(FILE *fp)
{
 return fwrite(&data[0][0], sizeof(data), 1, fp) ? true : false;
}

//-----------------------------------------------------------------------------
void Level::draw(int x, int y)
{
 // redraw the background
 screen->copyBoxFromContext(*bk, x*sprW + xOff, y*sprH + yOff, sprW, sprH);
 // draw the level square
 (*sprites)[blockSprites[data[x][y]]]->keep(screen, x*sprW + xOff, y*sprH + yOff);
}

//-----------------------------------------------------------------------------
void Level::draw()
/*
 * Draws the whole level.
 */
{
 for(int x=0; x<lvlW; x++)
  for(int y=0; y<lvlH; y++)
   draw(x, y);
}

//-----------------------------------------------------------------------------
void Level::setSquare(int x, int y, int type)
     /*
      * Sets a square. Also takes care of the dot count
      */
{
 if(BLOCK_IS_DOT(data[x][y])) dots--;
 data[x][y] = type;
 if(BLOCK_IS_DOT(data[x][y])) dots++;
 draw(x, y);
}

//-----------------------------------------------------------------------------
void Level::clear(int x, int y)
     /*
      * Clears one square of the level to BLOCK_BLANK
      */
{
 if(BLOCK_IS_DOT(data[x][y])) dots--;
 data[x][y] = BLOCK_BLANK;
 draw(x, y);
}

//-----------------------------------------------------------------------------
void Level::clear()
     /*
      * Clears the entire level
      */
{
 for(int x=0; x<lvlW; x++) for(int y=0; y<lvlH; y++) clear(x, y);
}

//-----------------------------------------------------------------------------
bool Level::checkLevel()
     /*
      * Checks that the level is valid - right number of special squares etc...
      */
{
 int nPlayer = 0, nBase = 0, nScore0 = 0, nScore2 = 0;
 for(int x=0; x<lvlW; x++) for(int y=0; y<lvlH; y++)
  {
   if(BLOCK_IS_START(data[x][y])) nPlayer++;
   if(BLOCK_IS_BASE(data[x][y])) nBase++;
   if(BLOCK_IS_SCORE_0(data[x][y])) nScore0++;
   if(BLOCK_IS_SCORE_2(data[x][y])) nScore2++;
  }
 if(nPlayer != 1) return false;
 if(nBase != 1) return false;
 if(nScore0 != 1) return false;
 if(nScore2 != 1) return false;
 return true;
}

//-----------------------------------------------------------------------------
Pos Level::find(int block)
{
 for(int x=0; x<lvlW; x++) for(int y=0; y<lvlH; y++)
  if(data[x][y] == block) return Pos(x, y);
 return Pos(-1, -1);
}
