/*C* $Id: menu.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "src/animation.h"
#include "src/data.h"
#include "src/menu.h"
#include "src/screen.h"
#include "src/ticker.h"
#include "util/util.h"
#include "vga++/font.h"
#include "vga++/keycodes.h"
#include "vga++/keys.h"
#include "vga++/vga++blur.h"

MenuItemType::MenuItemType(const char* Name, bool (*Callback)())
{
 name = newString(Name);
 callback = Callback; 
}

MenuItemType::~MenuItemType()
{
 delete name; 
}

MenuType::MenuType(void (*Callback)())
{
 items = new Collection<MenuItemType>(32, 32);
 callback = Callback;
}

MenuType::~MenuType()
{
 delete items;
}

void MenuType::Add(const char* Name, bool (*Callback)())
{
 items->add(new MenuItemType(Name, Callback));
}

void MenuType::Draw(int x, int y, int w, int h, int sel, VgaContext* bk)
{
 screen->copyBoxFromContext(*bk, x, y, w, h);
 int _y = y;
 fontBig->masked();
 fontBig->color(makeColor(5, 5, 5));
 fontBig->setHAlign(Font::Centre);
 for(int i=0; i<*items; i++)
  {
   if(i == sel) { screen->box(x, _y, w, fontBig->height(), makeColor(0, 0, 4)); }
   const char* s = *((*items)[i]);
   fontBig->writef(screen, x + (w >> 1), _y, s);
   _y += fontBig->height();
  }
 screen->update(x, y, w, h);
}

void MenuType::operator () (int x, int y, int w, int h, VgaContext* bk)
{
 keyboard->setMode(kmode_repeat);
 keyboard->waitRelease();
 int i = 0;
 callback();
 Draw(x, y, w, h, i, bk);
 Ticker ticker(3000 / animHat->getFrames());
 while(1)
  {
   if(ticker.past())
   {
    animHat->advanceFrame();
    animHat->blur(screen, (vgaW - animHat->getWidth()) / 2, 0); screen->blur();
    ticker.tickAndCatchUp();
   }
   if(keyboard->update())
    {
     KP_UP if(i>0) { i--; Draw(x, y, w, h, i, bk); }
     KP_DOWN if((i+1)<*items) { i++; Draw(x, y, w, h, i, bk); }
     KP_ENTER 
      {
       if(!(*(*items)[i])()) return;
       callback();
       Draw(x, y, w, h, i, bk);
       keyboard->setMode(kmode_repeat);
       keyboard->waitRelease();
      }
      KP_ESCAPE return;
      KP_Q return;
    }
  }
}
