
#include "src/options.h"
#include "util/util.h"
#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>

static Options staticOptions;
Options* options = &staticOptions;

static const char optionsHelp[] =
" -d, --dir      Specify where to find library files\n"
" -h, --help     This help screen\n"
" -l, --level    Specify level/sprite to play/edit immediately\n"
#ifdef SOUND
" -q, --nosound  Disable sound effects\n"
#endif
" -L, --ledit    Level editor mode\n"
" -S, --sedit    Sprite editor mode\n"
;

bool parseOptions(int argc, char* const argv[])
{
 options->level = NULL;
 options->libDir = LIBDIR;
 options->mode = Options::modePlay;
#ifdef SOUND
 options->sound = true;
#endif
 while(1)
  {
   int optionIndex;
   static struct option longOptions[] =
    {
     { "dir",     1, 0, 'd' },
     { "help",    0, 0, 'h' },
     { "level",   1, 0, 'l' },
     { "nosound", 0, 0, 'q' },
     { "ledit",   0, 0, 'L' },
     { "sedit",   0, 0, 'S' },
     { NULL,      0, 0, 0   }
    };
   int c;
   if((c = getopt_long(argc, argv, "d:hl:qLS", longOptions, &optionIndex)) == -1) return true;
   switch(c)
    {
    case 'd':
     if(options->libDir) delete options->libDir;
     options->libDir = newString(optarg);
     break;

    case 'h':
     printf("%s", optionsHelp);
     return false;
     
    case 'l':
     if(options->level) delete options->level;
     options->level = newString(optarg);
     break;
     
#ifdef SOUND
    case 'q':
     options->sound = false;
     break;
#endif
     
    case 'L':
     if(options->mode) { fprintf(stderr, "Only one of -L, -S, --ledit, --sedit can be specified.\n"); return false; }
     options->mode = Options::modeLevel;
     break;
     
    case 'S':
     if(options->mode) { fprintf(stderr, "Only one of -L, -S, --ledit, --sedit can be specified.\n"); return false; }
     options->mode = Options::modeSprite;
     break;
     
    case '?':
    default:
     printf("Invalid option.\n");
     return false;
    }
  }
}

