/*C* $Id: params.cc,v 1.4 1997/08/12 15:28:08 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "src/game.h"
#include "src/params.h"
#include "util/error.h"
#include "util/file.h"
#include "util/util.h"

LevelMode realDefaultLevelMode =
{
 NULL,
 0, 100, 40000,
 35, 25, 15, 100, 20,
 20,
 15000
};
LevelMode* defaultLevelMode = &realDefaultLevelMode;

Collection<LevelMode>* levelModes = NULL;

bool readParams(const char* filename)
{
 if(!levelModes) levelModes = new Collection<LevelMode>;
 else levelModes->delAll();

 File *f = new File(filename, "r");
 if(!*f) goto error;
 
 // read each line
 while(1)
  {
   // check for comments
   int c;
   while(c = f->getChar(), c == ' ' || c == '\t'); // trash whitespace
   if(c == ';' || c == '#' || c == '%') // accept any old comment character...
    {
     // skip over this line
     while(f->getChar() != '\n');
    }
   else
    {
     f->unGetChar(c);
     LevelMode* lm = new LevelMode;
     if(f->scanf(" %as %d %d %d %d %d %d %d %d %d %d \n", &lm->filename, &lm->fruit, &lm->fruitScore, &lm->fruitChance,
		 &lm->playerSpeed, &lm->ghostSpeed, &lm->ghostScaredSpeed, &lm->ghostEyesSpeed, &lm->fruitSpeed, &lm->mouthDelay,
		 &lm->ghostScaredTime) == 11)
      levelModes->add(lm);
     else
      {
       delete lm;
       break;
      }
    }
  }

 if(f->close()) goto error;

 delete f;
 return true;

error:
 setError("%s: %y", filename);
 delete f;
 return false;
}

bool runGame(const char* filename)
{
 if(!readParams(filename)) return false;
 game->setup();
 foreach(*levelModes, i)
  {
   LevelMode* lm = levelModes->at(i);
   if(!game->play(lm)) break;
  }
 return true;
}
