/*C* $Id: sound.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <strings.h>

#include "src/sound.h"
extern "C"
{
#include "src/soundIt.h"
}

const int SAMPLE_RATE = 11000;
const int PublicChannels = 4;
const int PrivateChannels = 1;

const int NumberOfSounds = 2;

/*
 * These store all the stuff about the sounds... This should probably be one
 * array of structs, actually... Oh no, maybe not... Hmm. That would probably
 * be too hard to define, perhaps.
 */
char *SoundNames[] = { "begin.raw", "credit.raw" };
Sample Sounds[NumberOfSounds]; 
bool SoundOk[NumberOfSounds];

//-----------------------------------------------------------------------------
Sound::Sound()
/*
 * Load samples and initialise the sounds driver. This should be done right at
 * the start of the program, to stop atexit functions being called from the
 * forked process... If any other processes require forking, a seperate 
 */
{
 sflag = true;

 printf("Loading sounds.\n");
 
 for(int i=0; i<NumberOfSounds; i++)
  {
   char* filename;
   asprintf(&filename, "%s/%s", SOUNDDIR, SoundNames[i]);
   if(!(SoundOk[i] = (Snd_loadRawSample(filename, &Sounds[i])? false : true)))
     printf("Error loading %s\n", filename); 
   free(filename);
  }

 if(Snd_init(NumberOfSounds, Sounds, SAMPLE_RATE, PublicChannels + PrivateChannels, "/dev/dsp") == -1)
  {
   printf("Cannot initialise sound - sorry, no sound\n");
   sflag = false;
  }
}

//-----------------------------------------------------------------------------
void Sound::Play(int effect, int Channel = -1)
/*
 * Play a sound, on a private, or public channel.
 */
{
 if(!sflag || !SoundOk[effect]) return;

 static int channel = 0;

 if(Channel < 0)
  {
   Snd_effect(effect, channel);
   channel++;
   channel %= PublicChannels;
  }
 else
  Snd_effect(effect, PublicChannels + Channel);
}

//-----------------------------------------------------------------------------
Sound::~Sound()
/*
 * Shut down the sound driver
 */
{
 if(!sflag) return;
 Snd_restore();
}

// Somewhere to store the pointer
Sound *Effects;

