/*C* $Id: sprites.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "src/sprites.h"
#include "util/file.h"
#include "util/util.h"
#include "vga++/sprite.h"

#include <errno.h>
#include <string.h>
#include <sys/stat.h>

Sprites::Sprites()
{
 data = new Collection<Sprite>(32, 32);
}

Sprites::~Sprites()
{
 delete data;
}

int Sprites::read(File* f, int w, int h)
{
 data->delAll();
 for(count=0; !f->eof(); count++)
  {
   Sprite *sp = new Sprite(w, h, *f);
   if(!sp->ok)
    if(f->eof()) { delete sp; break; }
    else
     {
      data->delAll();
      nonFatal("Error loading sprite %d from file.\n", count);
      return false;
     }
   data->add(sp);
  }
 return count? true : false;
}

int Sprites::load(File* f)
{
 int w, h;
 f->read(&w, sizeof(w));
 f->read(&h, sizeof(h));
 if(f->error())
  {
   nonFatal("Error reading sprite header.\n");
   return false;
  }
 return read(f, w, h);
}

bool Sprites::load(const char* filename)
{
 File* f = gzFilename(filename)? new GzipPipe(filename, "r") : new File(filename, "r");
 if(!*f) goto error;
 if(!load(f)) goto error;
 if(f->close()) goto error;
 delete f;
 return true;
error:
 delete f;
 nonFatal("Error reading from %s.\n", filename);
 return false;
}

bool Sprites::write(File* f)
{
 foreach(*data, i)
  {
   if(!(*data)[i]->write(*f))
    { nonFatal("Error writing to sprite file.\n"); return false; }
  }
 return true;
}

bool Sprites::save(File* f)
{
 int w = (*data)[0]->getWidth(), h = (*data)[0]->getHeight();
 f->write(&w, sizeof(w));
 f->write(&h, sizeof(h));
 if(f->error()) return false;
 return write(f);
}

bool Sprites::save(const char* filename)
{
 File* f = gzFilename(filename)? new GzipPipe(filename, "w") : new File(filename, "w");
 if(!*f) goto error;
 if(!save(f)) goto error;
 if(f->close()) goto error;
 delete f;
 return true;
error:
 delete f;
 nonFatal("Error writing to %s\n", filename);
 return false;
}

void Sprites::compile()
{
 every(*data, compile());
}

void Sprites::compileAndForget()
{
 every(*data, compileAndForget());
}


