/*C* $Id: file.cc,v 1.2 1997/08/11 19:21:31 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include "file.h"
#include <stdarg.h>
#include <stdlib.h>

File::File()
{
 fp = NULL;
}

File::File(const char* filename, const char* mode)
{
 fp = NULL;
 open(filename, mode);
}

File::~File()
{
 close();
}

FILE* File::open(const char* filename, const char* mode)
{
 close();
 return fp = fopen(filename, mode);
}

int File::close()
{
 if(!fp) return 0;
 int ret = fclose(fp);
 fp = NULL;
 return ret;
}

int File::scanf(const char* format, ...)
{
 va_list ap;
 va_start(ap, format);
 int ret = vfscanf(fp, format, ap);
 va_end(ap);
 return ret;
}

Pipe::Pipe()
 : File()
{
}

Pipe::Pipe(const char* filename, const char* mode)
{
 open(filename, mode);
}

Pipe::~Pipe()
{
 close();
}

FILE* Pipe::open(const char* filename, const char* mode)
{
 close();
 return fp = popen(filename, mode);
}

int Pipe::close()
{
 if(!fp) return 0;
 int ret = pclose(fp);
 fp = NULL;
 return ret;
}

GzipPipe::GzipPipe(const char* filename, const char* mode)
{
 open(filename, mode);
}

FILE* GzipPipe::open(const char* filename, const char* mode)
{
 if(!strcmp(mode, "r"))
  {
   char* s;
   asprintf(&s, "gzip -dc %s", filename);
   Pipe::open(s, "r"); // assigns fp
   free(s);
   return fp;
  }
 else if(!strcmp(mode, "w"))
  {
   char* s;
   asprintf(&s, "/bin/sh -c \"gzip - > %s\"", filename);
   Pipe::open(s, "w"); // assigns fp
   free(s);
   return fp;
  }
 else return NULL;
}
