/*C* $Id: font.h,v 1.2 1997/08/11 19:22:18 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#ifndef vgaxx_font_h
#define vgaxx_font_h

#include <stdarg.h>
#include <stdlib.h>
#include <vgagl.h>

class Font
{
public:
 enum HAlign { Left, Centre, Right };
 enum VAlign { Top, Middle, Bottom };

 Font();
 Font(unsigned char* _data, int _w, int _h, bool expanded = true, int _fg = makeColor(5, 5, 5));
 virtual ~Font();

 bool load(const char* filename);

 void restoreAndWritefAndUpdate(class BlurryVga* vc, int x, int y, const char* format, ...);
 void writef(class VgaContext* vc, int x, int y, const char* format, ...);
 void writefAndBlur(class BlurryVga* vc, int x, int y, const char* format, ...);
 void writefAndUpdate(class BlurryVga* vc, int x, int y, const char* format, ...);
 
 void overwrite() { wm &= ~WRITEMODE_MASKED; wm |= WRITEMODE_OVERWRITE; current = NULL; }
 void masked() { wm &= ~WRITEMODE_OVERWRITE; wm |= WRITEMODE_MASKED; current = NULL; }
 void color(int Fg, int Bg = 0) { fg = Fg; bg = Bg; current = NULL; }
 void setHAlign(HAlign _hAlign) { hAlign = _hAlign; }
 void setVAlign(VAlign _vAlign) { vAlign = _vAlign; }
 
 void select();
 
 int width() { return w; }
 virtual int width(const char* s);
 int widthf(const char* format, ...);
 int height() { return h; }

protected:
 void alignCoords(int& x, int& y, int l);
 int w, h;
 unsigned char* data;
 virtual void write(class VgaContext* vc, int x, int y, const char* s);

private:
 static Font* current = NULL;
 int wm, fg, bg;
 HAlign hAlign;
 VAlign vAlign;
};

class PropFont : public Font
{
public:
 PropFont() : Font() {};
 PropFont(unsigned char* _data, int _w, int _h, bool expanded = true);

 bool load(const char* filename);

 virtual int width(const char* s);

protected:
 virtual void write(class VgaContext* vc, int _x, int _y, const char* s);

private:
 char charWidths[256];
 void calcWidths();
};

extern Font defaultFont;

#endif



