/*C* $Id: vga++.cc,v 1.2 1997/08/11 19:22:18 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#include <errno.h>
#include <stdio.h>
#include <sys/stat.h>
#include "vga++.h"
#include "util/suid.h"
#include "util/util.h"

GraphicsContext* VgaContext::currentContext = NULL;

//-----------------------------------------------------------------------------
VgaContext::VgaContext()
{
 context = gl_allocatecontext();
}

//-----------------------------------------------------------------------------
VgaContext::~VgaContext()
{
 gl_freecontext(context);
}

//-----------------------------------------------------------------------------
void VgaContext::setPhysical()
{
 gl_setcontextvga(vgaMode); // invalidates currentContext...
 getContext();              // ...restores it
}

//-----------------------------------------------------------------------------
void VgaContext::setVirtual()
{
 gl_setcontextvgavirtual(vgaMode); // invalidates currentContext...
 getContext();                     // ...restores it
}

//-----------------------------------------------------------------------------
void Vga::initMode(bool AllowPaged = false)
{
 // Set the screen mode
 vga_setmode(vgaMode);
 
 // Initialise linear memory if possible
#ifdef VGALINEAR
 int linSize;
 suid_enable(); // that nasty svgalib... disgraceful behaviour
 linear = ((linSize=vga_setlinearaddressing()) == -1)?false:true;
 suid_disable();
#endif

 // Setup physical context
 physicalContext.setPhysical();

 // Check if video memory paging is possible (requires linear framebuffer)
#ifdef VGAPAGED
 vga_modeinfo* Info = vga_getmodeinfo(VGA_MODE);
 Paged = (AllowPaged && Linear && (LinSize > ((PageSize = Info->linewidth * Info->height) * 2)))? true:false;
#else
 (void)AllowPaged; // Prevent compiler warning
#endif

 // Initailise paged memory / virtual framebuffer
#ifdef VGAPAGED
 if (Paged)
  {
   WritePage = 1;
   physical.Select();
   GetContext();
   VBuf(PageSize);
  }
 else setVirtual();
#else
 setVirtual();
#endif

 // Output a little message. Oh yes.
 printf("Vga++");
#ifdef VGALINEAR
 if (linear) printf(" Linear");
#endif
#ifdef VGAPAGED
 if (paged) printf(" Paged");
#endif
 printf(" initialised.\n");
 fflush(stdout);
}

//-----------------------------------------------------------------------------
void Vga::update()
/*
 * Update the screen. In the case of Paged screen buffering, this actually
 * alters the Screen context's vbuf address.. Perhaps not the nicest way, but
 * it works, and it is fast, and requires no change to anything else.
 */
{
 select();
 vga_waitretrace();
#ifdef VGAPAGED
 if (Paged)
  {
   SetDisplayStart(0, WritePage? h() : 0);
   WritePage ^= 1;
   VBuf(WritePage? PageSize : -PageSize);
  }
 else
   copyScreen(physicalContext);
#else
 copyScreen(physicalContext);
#endif
}

//-----------------------------------------------------------------------------
#define PK __attribute__ ((packed));
typedef unsigned char uchar;
typedef unsigned short ushort;
typedef signed short sshort;
struct t_tga_header
{
 uchar id_size PK;
 uchar map_type PK;
 uchar image_type PK;
 ushort colormap_origin PK;
 ushort colormap_length PK;
 uchar colormap_entry_size PK;
 sshort image_x_origin PK;
 sshort image_y_origin PK;
 ushort image_width PK;
 ushort image_height PK;
 uchar image_pixel_size PK;
 uchar image_descriptor PK;
};
#undef PK

//-----------------------------------------------------------------------------
void Vga::screenShot()
{
 char s[128];
 static int imageNum = 0;
 for(; imageNum<9999; imageNum++)
  {
   sprintf(s, "screen%.4d.tga", imageNum);
   struct stat buf;
   if(stat(s, &buf) == -1) break;
  }
 FILE* fp = fopen(s, "w");
 if(!fp) return;
 printf("Saving screen shot into %s\n", s);

 t_tga_header header;
 
 // initialise the headers
 header.id_size = 0;
 header.map_type = 0;
 header.image_type = 2;

 header.colormap_origin = 0;
 header.colormap_length = 0;
 header.colormap_entry_size = 0;

 header.image_x_origin = 0;
 header.image_y_origin = 0;
 header.image_width = vgaW;
 header.image_height = vgaH;
 header.image_pixel_size = 24;
 header.image_descriptor = 0x20;
 
 // output the headers
 fwrite(&header, 18, 1, fp);
 
 // get the palette
 Palette pal;
 gl_getpalette(&pal);

 char data[vgaW * vgaH];
 physicalContext.select();
 gl_getbox(0, 0, vgaW, vgaH, data);

 // output the image
 char fileData[vgaW * vgaH * 3];
 char* p = data;
 char* q = fileData;
 for(int i=0; i<(vgaW * vgaH); i++)
  {
   unsigned char c = *p++;
   *q++ = pal.color[c].blue << 2;
   *q++ = pal.color[c].green << 2;
   *q++ = pal.color[c].red << 2;
  }
 fwrite(fileData, sizeof(fileData), 1, fp);

 fclose(fp);
}

//-----------------------------------------------------------------------------
void VgaContext::grid(int x1, int y1, int sx, int sy, int dx, int dy, int c)
{
 int px = x1, py = y1;
 for(int x = 0; x <= sx; x++, px+=dx) line(px, y1, px, y1+sy*dy, c);
 for(int y = 0; y <= sy; y++, py+=dy) line(x1, py, x1+sx*dx, py, c);
}

//-----------------------------------------------------------------------------
void VgaContext::rect(int x, int y, int w, int h, int c)
{ 
 int x1 = x, x2 = x1 + w - 1;
 int y1 = y, y2 = y1 + h - 1;
 line(x1, y1, x2, y1, c);
 line(x1, y2, x2, y2, c);
 line(x1, y1, x1, y2, c);
 line(x2, y1, x2, y2, c);
}
