/*C* $Id: vga++.h,v 1.2 1997/08/11 19:22:18 james Exp $
 *
 * Hatman - The Game of Kings
 * Copyright (C) 1997 James Pharaoh & Timothy Fisken
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *C*/

#ifndef vgaxx_h
#define vgaxx_h

#include <vga.h>
#include <vgagl.h>
#include <vgakeyboard.h>

#include "util/collection.h"
#include "vga++/mode.h"

class VgaContext
{
public:
 VgaContext();
 ~VgaContext();
 void setPhysical();
 void setVirtual();
 inline void select() { if(currentContext != context) setContext(); }

 inline void getContext() { gl_getcontext(currentContext = context); }
 inline void setContext() { gl_setcontext(currentContext = context); }

 void setDisplayStart(int x, int y) { gl_setdisplaystart(x, y); }
 void changeVBuf (size_t offset) { context->vbuf += offset; setContext(); }

 inline void box(int x, int y, int w, int h, int c)
  { select(); gl_fillbox(x, y, w, h, c); }
 inline void clear(int c = 0)
  { select(); gl_clearscreen(c); }
 inline void copyBoxFromContext(GraphicsContext* gc, int x, int y, int w, int h, int dx, int dy)
  { select(); gl_copyboxfromcontext(gc, x, y, w, h, dx, dy); }
 inline void copyBoxFromContext(GraphicsContext* gc, int x, int y, int w, int h)
  { select(); gl_copyboxfromcontext(gc, x, y, w, h, x, y); }
 inline void copyBoxToContext(GraphicsContext* gc, int x, int y, int w, int h, int dx, int dy)
  { select(); gl_copyboxtocontext(x, y, w, h, gc, dx, dy); }
 inline void copyBoxToContext(GraphicsContext* gc, int x, int y, int w, int h)
  { select(); gl_copyboxtocontext(x, y, w, h, gc, x, y); }
 inline void copyScreen(GraphicsContext* gc)
  { select(); gl_copyscreen(gc); }
 inline void fillBox(int x, int y, int w, int h, int c)
  { select(); gl_fillbox(x, y, w, h, c); }
 inline int getPixel(int x, int y)
  { select(); return gl_getpixel(x, y); }
 inline void line(int x1, int y1, int x2, int y2, int c)
  { select(); if(y1 == y2) gl_hline(x1, y1, x2, c); else gl_line(x1, y1, x2, y2, c); }
 inline void putBox(int x, int y, int w, int h, void *dp)
  { select(); gl_putbox(x, y, w, h, dp); }
 inline void putBoxMask(int x, int y, int w, int h, const char* dp)
  { select(); gl_putboxmask(x, y, w, h, (char*)dp); }
 inline void putBoxMaskCompiled(int x, int y, int w, int h, const char *dp)
  { select(); gl_putboxmaskcompiled(x, y, w, h, (char*) dp); }
 inline void putOnScreen(const char *c)
  { select(); gl_putbox(0, 0, vgaW, vgaH, (char*)c); }
 inline void setPixel(int x, int y, int c)
  { select(); gl_setpixel(x, y, c); }
 inline void write(int x, int y, const char *s)
  { select(); gl_write(x, y, (char*)s); }

 void grid(int x1, int y1, int sx, int sy, int dx, int dy, int c);
 void rect(int x, int y, int w, int h, int c);
 inline void rect(int x, int y, int w, int h, int c, int fill)
  {
   rect(x, y, w, h, c);
   box(x + 1, y + 1, w - 1, h - 1, fill);
  }

 void clip(int x1 = 0, int y1 = 0, int x2 = vgaW, int y2 = vgaH)
  { select(); gl_setclippingwindow(x1, y1, x2, y2); }
 void enableClipping()
  { select(); clipping = true; gl_enableclipping(); }
 void disableClipping()
  { select(); clipping = true; gl_disableclipping(); }
 bool isClipping() { return clipping; }

 int GetBytewidth() { return context->bytewidth; }
 char* GetVbuf() { return context->vbuf; }

 inline operator GraphicsContext* () { return context; }
protected:
 GraphicsContext* context;
private:
 static GraphicsContext* currentContext;
 bool clipping;
};

class Vga : public VgaContext
{
public:
 Vga() { vga_init(); }
 ~Vga() { textMode(); }
 
 void initMode(bool AllowPaged = false);
 void textMode() { vga_setmode(TEXT); }

 void update();
 
 void flip() { vga_flip(); }

 void clip(int x1 = 0, int y1 = 0, int x2 = vgaW, int y2 = vgaH)
  { VgaContext::clip(x1, y1, x2, y2); physicalContext.clip(x1, y1, x2, y2); }
 void enableClipping()
  { VgaContext::enableClipping(); physicalContext.enableClipping(); }
 void disableClipping()
  { VgaContext::disableClipping(); physicalContext.disableClipping(); }

 void screenShot();
protected:
 VgaContext physicalContext;
private:
#ifdef VGALINEAR
 bool linear;
#endif
#ifdef VGAPAGED
 bool paged;
 int pageSize;
 int writePage;
#endif
};

#endif
